' ========================================================================================
' GdipGetStringFormatMeasurableCharacterRangeCount
' ========================================================================================

' SED_PBWIN
#COMPILE EXE
#DIM ALL
#INCLUDE "GDIPLUS.INC"

' ========================================================================================
' The following example defines three ranges of character positions within a string and
' sets those ranges in a StringFormat object. Next, the GdipGetMeasurableCharacterRangeCount
' function is used to get the number of character ranges that are currently set in the
' StringFormat object. This number is then used to allocate a buffer large enough to store
' the regions that correspond with the ranges. Then, the GdipMeasureCharacterRanges function
' is used to get the three regions of the display that are occupied by the characters that
' are specified by the ranges.
' ========================================================================================
SUB GDIP_GetStringFormatMeasurableCharacterRangeCount (BYVAL hdc AS DWORD)

   LOCAL hStatus AS LONG
   LOCAL pGraphics AS DWORD
   LOCAL pBlueBrush AS DWORD
   LOCAL pRedBrush AS DWORD
   LOCAL pBlackPen AS DWORD
   LOCAL pFont AS DWORD
   LOCAL pFontFamily AS DWORD
   LOCAL pStringFormat AS DWORD
   LOCAL strFontName AS STRING
   LOCAL layoutRect AS RECTF

   hStatus = GdipCreateFromHDC(hdc, pGraphics)

   ' // Brushes and pens used for drawing and painting
   hStatus = GdipCreateSolidFill(GDIP_ARGB(255, 0, 0, 255), pBlueBrush)
   hStatus = GdipCreateSolidFill(GDIP_ARGB(100, 255, 0, 0), pRedBrush)
   hStatus = GdipCreatePen1(GDIP_ARGB(255, 0, 0, 0), 1, %UnitPixel, pBlackPen)

   ' // Three ranges of character positions within the string
   DIM charRanges(2) AS CharacterRange
   charRanges(0).First = 3  : charRanges(0).Length = 5
   charRanges(1).First = 15 : charRanges(1).Length = 2
   charRanges(2).First = 30 : charRanges(2).Length = 15

   ' // Font and string format used to apply to string when drawing
   strFontName = UCODE$("Times New Roman")
   hStatus = GdipCreateFontFamilyFromName(STRPTR(strFontName), %NULL, pFontFamily)
   IF hStatus = %StatusOk AND pFontFamily <> %NULL THEN
      hStatus = GdipCreateFont(pFontFamily, 16.0!, %FontStyleRegular, %UnitPoint, pFont)
      GdipDeleteFontFamily(pFontFamily)
   END IF

   ' Note: You can use the wrapper function GdiPlusCreateFontFromName to create the font:
   ' hStatus = GdiPlusCreateFontFromName("Times New Roman"", 16.0!, %FontStyleRegular, %UnitPoint, pFont)

   hStatus = GdipCreateStringFormat(0, %LANG_NEUTRAL, pStringFormat)

   ' // Layout rectangle used for drawing string
   layoutRect.x = 20.0! : layoutRect.y = 20.0! : layoutRect.Width = 130.0! : layoutRect.Height = 130.0!

   ' // Other variables
   DIM   rgCharRangeRegions(0) AS DWORD   ' // array of CharacterRange regions
   LOCAL i AS LONG                        ' // loop counter
   LOCAL count AS LONG                    ' // number of character ranges set
   LOCAL strText AS STRING
   strText = UCODE$("The quick, brown fox easily jumps over the lazy dog.")

   ' // Set three ranges of character positions.
   hStatus = GdipSetStringFormatMeasurableCharacterRanges(pStringFormat, 3, charRanges(0))

   ' // Get the number of ranges that have been set, and allocate memory to
   ' // store the regions that correspond to the ranges.
   hStatus = GdipGetStringFormatMeasurableCharacterRangeCount(pStringFormat, count)

   REDIM rgCharRangeRegions(count - 1)
   FOR i = 0 TO count - 1
      hStatus = GdipCreateRegion(rgCharRangeRegions(i))
   NEXT

   ' // Get the regions that correspond to the ranges within the string.
   ' // Then draw the string and show the regions.
   hStatus = GdipMeasureCharacterRanges(pGraphics, STRPTR(strText), -1, _
      pFont, layoutRect, pStringFormat, count, rgCharRangeRegions(0))

   hStatus = GdipDrawString(pGraphics, STRPTR(strText), -1, _
      pFont, layoutRect, pStringFormat, pBlueBrush)
   hStatus = GdipDrawRectangle(pGraphics, pBlackPen, layoutRect.x, layoutRect.y, layoutRect.Width, layoutRect.Height)

   FOR i = 0 TO count - 1
      hStatus = GdipFillRegion(pGraphics, pRedBrush, rgCharRangeRegions(i))
   NEXT

   FOR i = 0 TO count - 1
      IF rgCharRangeRegions(i) THEN GdipDeleteRegion(rgCharRangeRegions(i))
   NEXT

   ' // Cleanup
   IF pFont THEN GdipDeleteFont(pFont)
   IF pStringFormat THEN GdipDeleteStringFormat(pStringFormat)
   IF pBlackPen THEN GdipDeletePen(pBlackPen)
   IF pBlueBrush THEN GdipDeleteBrush(pBlueBrush)
   IF pRedBrush THEN GdipDeleteBrush(pRedBrush)
   IF pGraphics THEN GdipDeleteGraphics(pGraphics)

END SUB
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
FUNCTION WINMAIN (BYVAL hInstance AS DWORD, BYVAL hPrevInstance AS DWORD, BYVAL lpszCmdLine AS ASCIIZ PTR, BYVAL nCmdShow AS LONG) AS LONG

   LOCAL hr AS LONG
   LOCAL hWndMain AS DWORD
   LOCAL hCtl AS DWORD
   LOCAL hFont AS DWORD
   LOCAL wcex AS WndClassEx
   LOCAL szClassName AS ASCIIZ * 80
   LOCAL rc AS RECT
   LOCAL szCaption AS ASCIIZ * 255
   LOCAL nLeft AS LONG
   LOCAL nTop AS LONG
   LOCAL nWidth AS LONG
   LOCAL nHeight AS LONG
   LOCAL token AS DWORD
   LOCAL StartupInput AS GdiplusStartupInput

   ' Initialize GDI+
   StartupInput.GdiplusVersion = 1
   hr = GdiplusStartup(token, StartupInput, BYVAL %NULL)
   IF hr THEN
      MSGBOX "Error initializing GDI+"
      EXIT FUNCTION
   END IF

   hFont = GetStockObject(%ANSI_VAR_FONT)

   ' Register the window class
   szClassName        = "MyClassName"
   wcex.cbSize        = SIZEOF(wcex)
   wcex.style         = %CS_HREDRAW OR %CS_VREDRAW
   wcex.lpfnWndProc   = CODEPTR(WndProc)
   wcex.cbClsExtra    = 0
   wcex.cbWndExtra    = 0
   wcex.hInstance     = hInstance
   wcex.hCursor       = LoadCursor (%NULL, BYVAL %IDC_ARROW)
   wcex.hbrBackground = GetStockObject(%WHITE_BRUSH)
   wcex.lpszMenuName  = %NULL
   wcex.lpszClassName = VARPTR(szClassName)
   wcex.hIcon         = LoadIcon (%NULL, BYVAL %IDI_APPLICATION) ' Sample, if resource icon: LoadIcon(hInst, "APPICON")
   wcex.hIconSm       = LoadIcon (%NULL, BYVAL %IDI_APPLICATION) ' Remember to set small icon too..
   RegisterClassEx wcex

   ' Window caption
   szCaption = "GdipGetStringFormatMeasurableCharacterRangeCount"

   ' Retrieve the nSize of the working area
   SystemParametersInfo %SPI_GETWORKAREA, 0, BYVAL VARPTR(rc), 0

   ' Calculate the position and nSize of the window
   nWidth  = (((rc.nRight - rc.nLeft)) + 2) * 0.50   ' 50% of the client screen width
   nHeight = (((rc.nBottom - rc.nTop)) + 2) * 0.50   ' 50% of the client screen height
   nLeft   = ((rc.nRight - rc.nLeft) \ 2) - nWidth \ 2
   nTop    = ((rc.nBottom - rc.nTop) \ 2) - (nHeight \ 2)

   ' Create a window using the registered class
   hWndMain = CreateWindowEx(%WS_EX_CONTROLPARENT, _           ' extended style
                             szClassName, _                    ' window class name
                             szCaption, _                      ' window caption
                             %WS_OVERLAPPEDWINDOW OR _
                             %WS_CLIPCHILDREN, _               ' window style
                             nLeft, _                          ' initial x position
                             nTop, _                           ' initial y position
                             nWidth, _                         ' initial x nSize
                             nHeight, _                        ' initial y nSize
                             %NULL, _                          ' parent window handle
                             0, _                              ' window menu handle
                             hInstance, _                      ' program instance handle
                             BYVAL %NULL)                      ' creation parameters

   ' Show the window
   ShowWindow hWndMain, nCmdShow
   UpdateWindow hWndMain

   ' Message handler loop
   LOCAL Msg AS tagMsg
   WHILE GetMessage(Msg, %NULL, 0, 0)
      IF ISFALSE IsDialogMessage(hWndMain, Msg) THEN
         TranslateMessage Msg
         DispatchMessage Msg
      END IF
   WEND

   ' Shutdown GDI+
   GdiplusShutdown token

   FUNCTION = msg.wParam

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main Window procedure
' ========================================================================================
FUNCTION WndProc (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, BYVAL wParam AS DWORD, BYVAL lParam AS LONG) AS LONG

   LOCAL hDC AS DWORD
   LOCAL ps AS PAINTSTRUCT

   SELECT CASE wMsg

      CASE %WM_COMMAND
         SELECT CASE LOWRD(wParam)
            CASE %IDCANCEL
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  SendMessage hWnd, %WM_CLOSE, 0, 0
                  FUNCTION = 0
                  EXIT FUNCTION
               END IF
         END SELECT

      CASE %WM_SYSCOMMAND
         ' Capture this message and send a WM_CLOSE message
         IF (wParam AND &HFFF0) = %SC_CLOSE THEN
            SendMessage hWnd, %WM_CLOSE, 0, 0
            EXIT FUNCTION
         END IF

      CASE %WM_PAINT
         hDC = BeginPaint(hWnd, ps)
         GDIP_GetStringFormatMeasurableCharacterRangeCount hDC
         EndPaint(hWnd, ps)

      CASE %WM_DESTROY
         PostQuitMessage 0
         EXIT FUNCTION

   END SELECT

   FUNCTION = DefWindowProc(hWnd, wMsg, wParam, lParam)

END FUNCTION
' ========================================================================================
